@@ -1,5 +1,7 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
+#include <linux/if_arp.h>
+
 #include "missing_network.h"
 #include "xfrm.h"
 
@@ -17,11 +19,11 @@ static int xfrm_fill_message_create(NetDev *netdev, Link *link, sd_netlink_messa
 
         r = sd_netlink_message_append_u32(message, IFLA_XFRM_LINK, link ? link->ifindex : LOOPBACK_IFINDEX);
         if (r < 0)
-                return log_netdev_error_errno(netdev, r, "Could not append IFLA_XFRM_LINK: %m");
+                return r;
 
         r = sd_netlink_message_append_u32(message, IFLA_XFRM_IF_ID, x->if_id);
         if (r < 0)
-                return log_netdev_error_errno(netdev, r, "Could not append IFLA_XFRM_IF_ID: %m");
+                return r;
 
         return 0;
 }
@@ -48,5 +50,6 @@ const NetDevVTable xfrm_vtable = {
         .sections = NETDEV_COMMON_SECTIONS "Xfrm\0",
         .fill_message_create = xfrm_fill_message_create,
         .config_verify = xfrm_verify,
-        .create_type = NETDEV_CREATE_STACKED
+        .create_type = NETDEV_CREATE_STACKED,
+        .iftype = ARPHRD_NONE,
 };
