@@ -129,7 +129,7 @@ int socket_address_parse(SocketAddress *a, const char *s) {
                                         .size = sizeof(struct sockaddr_in6),
                                 };
                         else
-                                assert_not_reached("Family quarrel");
+                                assert_not_reached();
                 }
         }
 
@@ -226,7 +226,7 @@ int make_socket_fd(int log_level, const char* address, int type, int flags) {
 
         a.type = type;
 
-        fd = socket_address_listen(&a, type | flags, SOMAXCONN, SOCKET_ADDRESS_DEFAULT,
+        fd = socket_address_listen(&a, type | flags, SOMAXCONN_DELUXE, SOCKET_ADDRESS_DEFAULT,
                                    NULL, false, false, false, 0755, 0644, NULL);
         if (fd < 0 || log_get_max_level() >= log_level) {
                 _cleanup_free_ char *p = NULL;
